"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _semver = _interopRequireDefault(require("semver"));

var _inquirer = _interopRequireDefault(require("inquirer"));

var _storyboard = require("storyboard");

var _readSpecs = require("./utils/readSpecs");

var _writeSpecs = _interopRequireDefault(require("./utils/writeSpecs"));

var _shell = require("./utils/shell");

var _git = require("./utils/git");

var _changelog = require("./utils/changelog");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const DEBUG_SKIP_CHECKS = false;
const RELEASE_INCREMENTS = ['major', 'minor', 'patch'];
const PRERELEASE_INCREMENTS = ['rc', 'beta', 'alpha'];
const INCREMENTS = [...RELEASE_INCREMENTS, ...PRERELEASE_INCREMENTS];

const run =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(function* ({
    src,
    ignoreSrc,
    master,
    checkUncommitted,
    checkUnpulled,
    checks,
    confirm,
    gitCommit,
    newVersion,
    npmPublish,
    publishTag,
    incrementVersionBy,
    changelog,
    changelogPath,
    single,
    _date,
    _masterVersion
  }) {
    const allSpecs = yield (0, _readSpecs.readAllSpecs)(src, ignoreSrc); // Confirm that we have run build

    if (confirm) {
      const _ref2 = yield _inquirer.default.prompt([{
        name: 'confirmBuild',
        type: 'confirm',
        message: 'Have you built all your packages for production?',
        default: false
      }]),
            confirmBuild = _ref2.confirmBuild;

      if (!confirmBuild) return;
    } // Prepublish checks


    if (checks) {
      yield prepublishChecks({
        master,
        checkUncommitted,
        checkUnpulled
      });
    } // Get last tag and find packages requiring updates


    const lastTag = yield (0, _git.gitLastTag)();
    const dirty = yield findPackagesToUpdate(allSpecs, lastTag, single);

    if (!dirty.length) {
      _storyboard.mainStory.info('No packages have been updated');

      return;
    } // Determine a suitable version number


    const masterVersion = _masterVersion || (yield getMasterVersion(allSpecs, lastTag));
    if (masterVersion == null) return;

    if (incrementVersionBy) {
      if (INCREMENTS.indexOf(incrementVersionBy) < 0) {
        _storyboard.mainStory.error(`Value specified for --increment-version-by: ${_storyboard.chalk.bold(incrementVersionBy)} is invalid.`);

        _storyboard.mainStory.error(`It should be one of (${INCREMENTS.join(', ')}), or not specified.`);

        if (!DEBUG_SKIP_CHECKS) throw new Error('INVALID_INCREMENT_BY_VALUE');
      }
    }

    const nextVersion = newVersion || calcNextVersion(masterVersion, incrementVersionBy) || (yield promptNextVersion(masterVersion)); // Confirm before proceeding

    if (confirm) {
      const _ref3 = yield _inquirer.default.prompt([{
        name: 'confirmPublish',
        type: 'confirm',
        message: `Confirm release (${_storyboard.chalk.yellow.bold(dirty.length)} package/s, ` + `v${_storyboard.chalk.cyan.bold(nextVersion)})?`,
        default: false
      }]),
            confirmPublish = _ref3.confirmPublish;

      if (!confirmPublish) return;
    } // Update package.json's for dirty packages AND THE ROOT PACKAGE + changelog


    const dirtyPlusRoot = single ? dirty : dirty.concat(_readSpecs.ROOT_PACKAGE);

    for (let i = 0; i < dirtyPlusRoot.length; i++) {
      const pkgName = dirtyPlusRoot[i];
      const _allSpecs$pkgName = allSpecs[pkgName],
            specPath = _allSpecs$pkgName.specPath,
            specs = _allSpecs$pkgName.specs;
      specs.version = nextVersion;
      (0, _writeSpecs.default)(specPath, specs);
    }

    if (changelog) (0, _changelog.addVersionLine)({
      changelogPath,
      version: nextVersion,
      _date
    }); // Commit, tag and push

    if (gitCommit) {
      yield (0, _git.gitCommitChanges)(`v${nextVersion}`);
      yield (0, _git.gitAddTag)(`v${nextVersion}`);
      yield (0, _git.gitPushWithTags)();
    } // Publish


    if (npmPublish) {
      for (let i = 0; i < dirty.length; i++) {
        const pkgName = dirty[i];
        const _allSpecs$pkgName2 = allSpecs[pkgName],
              pkgPath = _allSpecs$pkgName2.pkgPath,
              specs = _allSpecs$pkgName2.specs;
        if (specs.private) continue; // we don't want npm to complain :)

        let cmd = 'npm publish';
        if (publishTag != null) cmd += ` --tag ${publishTag}`;
        yield (0, _shell.exec)(cmd, {
          cwd: pkgPath
        });
      }
    }
  });

  return function run(_x) {
    return _ref.apply(this, arguments);
  };
}(); // ------------------------------------------------
// Helpers
// ------------------------------------------------


const prepublishChecks =
/*#__PURE__*/
function () {
  var _ref4 = _asyncToGenerator(function* ({
    master,
    checkUncommitted,
    checkUnpulled
  }) {
    if (DEBUG_SKIP_CHECKS) {
      _storyboard.mainStory.warn('DEBUG_SKIP_CHECKS should be disabled!!');
    } // Check current branch


    const branch = yield (0, _git.gitCurBranch)();

    if (branch !== 'master') {
      if (master) {
        _storyboard.mainStory.error(`Can't publish from current branch: ${_storyboard.chalk.bold(branch)}`);

        if (!DEBUG_SKIP_CHECKS) throw new Error('BRANCH_CHECK_FAILED');
      }

      _storyboard.mainStory.warn(`Publishing from a non-master branch: ${_storyboard.chalk.red.bold(branch)}`);
    } else {
      _storyboard.mainStory.info(`Current branch: ${_storyboard.chalk.yellow.bold(branch)}`);
    } // Check that the branch is clean


    const uncommitted = yield (0, _git.gitUncommittedChanges)();

    if (uncommitted !== '') {
      if (checkUncommitted) {
        _storyboard.mainStory.error(`Can't publish with uncommitted changes (stash/commit them): \n${_storyboard.chalk.bold(uncommitted)}`);

        if (!DEBUG_SKIP_CHECKS) throw new Error('UNCOMMITTED_CHECK_FAILED');
      }

      _storyboard.mainStory.warn('Publishing with uncommitted changes');
    } else {
      _storyboard.mainStory.info('No uncommitted changes');
    } // Check remote history


    const unpulled = yield (0, _git.gitUnpulledChanges)();

    if (unpulled !== '0') {
      if (checkUnpulled) {
        _storyboard.mainStory.error('Remote history differs. Please pull changes');

        if (!DEBUG_SKIP_CHECKS) throw new Error('UNPULLED_CHECK_FAILED');
      }

      _storyboard.mainStory.warn('Publishing with unpulled changes');
    } else {
      _storyboard.mainStory.info('Remote history matches local history');
    }
  });

  return function prepublishChecks(_x2) {
    return _ref4.apply(this, arguments);
  };
}();

const findPackagesToUpdate =
/*#__PURE__*/
function () {
  var _ref5 = _asyncToGenerator(function* (allSpecs, lastTag, single) {
    const pkgNames = Object.keys(allSpecs);
    const dirty = [];

    for (let i = 0; i < pkgNames.length; i++) {
      const pkgName = pkgNames[i];
      if (pkgName === _readSpecs.ROOT_PACKAGE && !single) continue;
      const _allSpecs$pkgName3 = allSpecs[pkgName],
            pkgPath = _allSpecs$pkgName3.pkgPath,
            specs = _allSpecs$pkgName3.specs;
      const diff = yield (0, _git.gitDiffSinceIn)(lastTag, pkgPath);

      if (diff !== '') {
        const numChanges = diff.split('\n').length;

        _storyboard.mainStory.info(`- Package ${pkgName} (currently ${_storyboard.chalk.cyan.bold(specs.version)}) has changed (#files: ${numChanges})`);

        dirty.push(pkgName);
      }
    }

    return dirty;
  });

  return function findPackagesToUpdate(_x3, _x4, _x5) {
    return _ref5.apply(this, arguments);
  };
}();

const getMasterVersion =
/*#__PURE__*/
function () {
  var _ref6 = _asyncToGenerator(function* (allSpecs, lastTag) {
    let masterVersion = allSpecs[_readSpecs.ROOT_PACKAGE].specs.version;

    if (lastTag != null) {
      const tagVersion = _semver.default.clean(lastTag);

      _storyboard.mainStory.info(`Last tag found: ${_storyboard.chalk.yellow.bold(lastTag)}`);

      if (tagVersion !== masterVersion) {
        _storyboard.mainStory.warn(`Last tagged version ${_storyboard.chalk.cyan.bold(tagVersion)} does not match package.json version ${_storyboard.chalk.cyan.bold(masterVersion)}`);

        _storyboard.mainStory.warn('This may cause inaccuracies when determining which packages ' + 'need to be released, since oao uses tags to detect package changes');

        const _ref7 = yield _inquirer.default.prompt([{
          name: 'confirm',
          type: 'confirm',
          message: 'Continue?',
          default: false
        }]),
              confirm = _ref7.confirm;

        if (!confirm) return null;

        if (_semver.default.valid(tagVersion) && _semver.default.gt(tagVersion, masterVersion)) {
          masterVersion = tagVersion;
        }

        _storyboard.mainStory.warn(`Using ${_storyboard.chalk.cyan.bold(masterVersion)} as reference (the highest one of both)`);
      }
    } else {
      _storyboard.mainStory.warn('Repo has no tags yet');
    }

    if (!_semver.default.valid(masterVersion)) {
      _storyboard.mainStory.error(`Master version ${_storyboard.chalk.cyan.bold(masterVersion)} is invalid. Please correct it manually`);

      throw new Error('INVALID_VERSION');
    }

    return masterVersion;
  });

  return function getMasterVersion(_x6, _x7) {
    return _ref6.apply(this, arguments);
  };
}();

const calcNextVersion = (prevVersion, incrementBy = '') => {
  const isPreRelease = PRERELEASE_INCREMENTS.indexOf(incrementBy) >= 0;
  const increment = isPreRelease ? 'prerelease' : incrementBy;
  const isNewPreRelease = isPreRelease && prevVersion.indexOf(incrementBy) < 0;
  return isNewPreRelease ? `${_semver.default.inc(prevVersion, 'major')}-${incrementBy}.0` : _semver.default.inc(prevVersion, increment);
};

const promptNextVersion =
/*#__PURE__*/
function () {
  var _ref8 = _asyncToGenerator(function* (prevVersion) {
    const major = _semver.default.inc(prevVersion, 'major');

    const minor = _semver.default.inc(prevVersion, 'minor');

    const patch = _semver.default.inc(prevVersion, 'patch');

    const prerelease = _semver.default.inc(prevVersion, 'prerelease');

    const rc = prevVersion.indexOf('rc') < 0 ? `${major}-rc.0` : prerelease;
    const beta = prevVersion.indexOf('beta') < 0 ? `${major}-beta.0` : prerelease;
    const alpha = prevVersion.indexOf('alpha') < 0 ? `${major}-alpha.0` : prerelease;

    const _ref9 = yield _inquirer.default.prompt([{
      name: 'nextVersion',
      type: 'list',
      message: `Current version is ${_storyboard.chalk.cyan.bold(prevVersion)}. Next one?`,
      choices: [{
        name: `Major (${_storyboard.chalk.cyan.bold(major)})`,
        value: major
      }, {
        name: `Minor (${_storyboard.chalk.cyan.bold(minor)})`,
        value: minor
      }, {
        name: `Patch (${_storyboard.chalk.cyan.bold(patch)})`,
        value: patch
      }, {
        name: `Release candidate (${_storyboard.chalk.cyan.bold(rc)})`,
        value: rc
      }, {
        name: `Beta (${_storyboard.chalk.cyan.bold(beta)})`,
        value: beta
      }, {
        name: `Alpha (${_storyboard.chalk.cyan.bold(alpha)})`,
        value: alpha
      }],
      defaultValue: 2
    }]),
          nextVersion = _ref9.nextVersion;

    return nextVersion;
  });

  return function promptNextVersion(_x8) {
    return _ref8.apply(this, arguments);
  };
}(); // ------------------------------------------------
// Public
// ------------------------------------------------


var _default = run;
exports.default = _default;